/* Final Project - Baby Monitor
 *  
Baby Monitor is a hardware-software solution for babysitting.

Our baby monitor allows to supervise a couple of scenarios:

1. Noise Detection - Notify you when the baby is crying

2. Temperature Detection - Displays the current temperature in the baby's room and notifies when the temperature exceed 30 degrees or drops down below 20 degrees.

3. Find My Baby - If necessary, allows you to press a button that helps you find the baby by triggering noise

4. Play a song - Allows you to play a song to calm down the baby

* Created By:
- Yuval Karp 316519107
- Hila Mizrahi 315697185
 */

// enter these values 
#define BLYNK_TEMPLATE_ID           ""
#define BLYNK_DEVICE_NAME           ""
#define BLYNK_AUTH_TOKEN            ""

#define BLYNK_PRINT SerialUSB
#include <Adafruit_CircuitPlayground.h>
#include "pitches.h"


#include <ESP8266_Lib.h>
#include <BlynkSimpleShieldEsp8266.h>

// defining notes 
const int speed = 2;
const int green = 75; // brightness of the Neopixels... 
const int red   = 125;
const int numNotes3 = 26;

//notes 

int melody3[] = {                          
  NOTE_B4, NOTE_B4, NOTE_B4, 
  NOTE_B4, NOTE_B4, NOTE_B4,
  NOTE_B4, NOTE_D5, NOTE_G4, NOTE_A4,
  NOTE_B4, 0,
  NOTE_C5, NOTE_C5, NOTE_C5, NOTE_C5, 
  NOTE_C5, NOTE_B4, NOTE_B4, NOTE_B4, NOTE_B4, 
  NOTE_D5, NOTE_D5, NOTE_C5, NOTE_A4, 
  NOTE_G4};

int noteDurations3[] = {    // Duration for each note
  4, 4, 2, 
  4, 4, 2, 
  4, 4, 3, 8,
  1, 4,
  4, 4, 3, 8, 
  4, 4, 4, 8, 8,
  4, 4, 4, 4, 
  1
  };

// Modes
bool Find_My_Baby = false;
bool Find_Temp = false;
bool Baby_Cry = false;
bool Play_Song = false;
bool Response = false; 

uint8_t i,k;
int16_t Speed;
int Temp = CircuitPlayground.temperature();

#define ARRAY_LENGTH(arr) (sizeof(arr) / sizeof(*arr))

char auth[] = BLYNK_AUTH_TOKEN;

// enter WiFi credentials.
char ssid[] = "";
char pass[] = "";

int buttons[] = {V0,V1,V2, V3};

// resets all the modes 
void resetAll() {
  Find_My_Baby = false; 
  Response = false;
  Play_Song = false;

  CircuitPlayground.clearPixels();
  CircuitPlayground.playTone(0, 0);

  for (int i = 0; i < ARRAY_LENGTH(buttons); i++) {
    Blynk.virtualWrite(buttons[i], 0);
  }
}
// Hardware Serial on Mega, Leonardo, Micro...
#define EspSerial Serial1


// Your ESP8266 baud rate:
#define ESP8266_BAUD 115200

ESP8266 wifi(&EspSerial);

// This function will be called every time the "Find my baby" button will be pressed,
// Blynk app writes values to the Virtual Pin V0
BLYNK_WRITE(V0)
{
  int pinValue = param.asInt(); // assigning incoming value from pin V0 to a variable
  if (pinValue == 1){
      Find_My_Baby = true; 
   }
   else {
      Response = true; 
   }
  }
//This function will be called every time the "Play song" button will be pressed,
// Blynk app writes values to the Virtual Pin V3
BLYNK_WRITE(V3)
{
  int pinValue = param.asInt(); // assigning incoming value from pin V3 to a variable
  if (pinValue == 1){
      Play_Song = true; 
   }
   
  }
  
void setup()
{
  // Debug console
  SerialUSB.begin(115200);
  CircuitPlayground.begin();

  // Set ESP8266 baud rate
  EspSerial.begin(ESP8266_BAUD);
  delay(10);
  Speed = 250;
  Blynk.begin(auth, wifi, ssid, pass);
  
}


void loop()
{
  Blynk.run();
  int noise = CircuitPlayground.soundSensor();
  Serial.println(noise);
  Serial.println(Temp);
  
  Temp_Status();
  
  if (Find_My_Baby) {
    Find_My_Baby_Status();
  }
  if (Play_Song) {
    Play_Song_Status();
  }
  if (noise >= 9 && noise < 11 ){
    Baby_Cry_Status(5);
  }
  if (noise > 12){
    Baby_Cry_Status(10);
    Baby_Cry_notification("Your baby is crying!");
  }
  if (noise < 9){
    Baby_Cry_Status(0);
  }
}


void Find_My_Baby_Status() {
  for (int i = 0; i < 10; i++) {
       CircuitPlayground.setPixelColor(i, 255,   0,   0);
   }  
   if (Response) {
      resetAll();
  } else {
     CircuitPlayground.playTone(500, 0);
  }
}

void Temp_Status() {
  Temp = CircuitPlayground.temperature(); 
  Blynk.virtualWrite(V1,Temp);
  if (Temp <= 30 && Temp >= 20) {
      CircuitPlayground.clearPixels();
  }
  if (Temp > 30) {
    Blynk.virtualWrite(V4,"It is too hot in the baby's room!");
    for (int i = 0; i < 10; i++) {
       CircuitPlayground.setPixelColor(i, 255,   0,   0);
   }
   delay(2000);
   Temp = CircuitPlayground.temperature();
   Blynk.virtualWrite(V1,Temp);
   if (Temp <=30) {
    CircuitPlayground.clearPixels();
   }
  }
  if (Temp < 20) {
    Blynk.virtualWrite(V4,"It is too cold in the baby's room!");
    for (int i = 0; i < 10; i++) {
       CircuitPlayground.setPixelColor(i, 0,   0,   255);
   }
   delay(2000);
   Temp = CircuitPlayground.temperature();
   if (Temp >=20) {
    CircuitPlayground.clearPixels();
   }
  }
  if (Response) {
      resetAll();
  }
  }

void Baby_Cry_Status(int value) {
  Blynk.virtualWrite(V2,value);
}

void Baby_Cry_notification(String value) {
   Blynk.virtualWrite(V4,value);

}

void Play_Song_Status() {
  for (int p = 0; p < 10; p++){
    CircuitPlayground.setPixelColor(p, 0, 0, 0);
  }
  
    for (int thisNote = 0; thisNote < numNotes3; thisNote++) { // play notes of the melody
      if (melody3[thisNote] != 0){
        if (thisNote % 2){
          for (int p = 0; p <= 8; p=p+2){
            CircuitPlayground.setPixelColor(p, 255,20,147);
            CircuitPlayground.setPixelColor(p+1,0,32,255);
          }
        }
        else{
          for (int p = 0; p <= 8; p=p+2){
            CircuitPlayground.setPixelColor(p, 255,20,147);
            CircuitPlayground.setPixelColor(p+1,0,32,255);
          }
        }
      }
      
      int noteDuration = (1000 / speed) / noteDurations3[thisNote];
      CircuitPlayground.playTone(melody3[thisNote], noteDuration);

      int pauseBetweenNotes = noteDuration * 1.30;
      delay(pauseBetweenNotes);
    }
    CircuitPlayground.clearPixels();
    resetAll();
}
  
